//
// Make Russian index from TeX idx-file.
// Must be performed BEFORE makeindex!
// The program replaces the Russian letters,
// produced by TeX in the style "\IeC {\cyra }",
// by normal Russian letters in Windows CP-1251
// encoding. Windows is choosed, because
// the codes of Russian letters
// are ordered in Windows encoding.
//
#include <stdio.h>
#include <string.h>

static const char * const TeX_Russians[] = {
    // Upper case letters
    "\\IeC {\\CYRA }", "\\IeC {\\CYRB }", "\\IeC {\\CYRV }",
    "\\IeC {\\CYRG }", "\\IeC {\\CYRD }",
    "\\IeC {\\CYRE }", "\\IeC {\\CYRZH }", "\\IeC {\\CYRZ }",
    "\\IeC {\\CYRI }", "\\IeC {\\CYRISHRT }",
    "\\IeC {\\CYRK }", "\\IeC {\\CYRL }", "\\IeC {\\CYRM }",
    "\\IeC {\\CYRN }", "\\IeC {\\CYRO }",
    "\\IeC {\\CYRP }", "\\IeC {\\CYRR }", "\\IeC {\\CYRS }",
    "\\IeC {\\CYRT }", "\\IeC {\\CYRU }",
    "\\IeC {\\CYRF }", "\\IeC {\\CYRH }", "\\IeC {\\CYRC }",
    "\\IeC {\\CYRCH }", "\\IeC {\\CYRSH }",
    "\\IeC {\\CYRSHCH }", "\\IeC {\\CYRHRDSN }", "\\IeC {\\CYRERY }",
    "\\IeC {\\CYRSFTSN }", "\\IeC {\\CYREREV }",
    "\\IeC {\\CYRYU }", "\\IeC {\\CYRYA }",

    // Lower case letters
    "\\IeC {\\cyra }", "\\IeC {\\cyrb }", "\\IeC {\\cyrv }",
    "\\IeC {\\cyrg }", "\\IeC {\\cyrd }",
    "\\IeC {\\cyre }", "\\IeC {\\cyrzh }", "\\IeC {\\cyrz }",
    "\\IeC {\\cyri }", "\\IeC {\\cyrishrt }",
    "\\IeC {\\cyrk }", "\\IeC {\\cyrl }", "\\IeC {\\cyrm }",
    "\\IeC {\\cyrn }", "\\IeC {\\cyro }",
    "\\IeC {\\cyrp }", "\\IeC {\\cyrr }", "\\IeC {\\cyrs }",
    "\\IeC {\\cyrt }", "\\IeC {\\cyru }",
    "\\IeC {\\cyrf }", "\\IeC {\\cyrh }", "\\IeC {\\cyrc }",
    "\\IeC {\\cyrch }", "\\IeC {\\cyrsh }",
    "\\IeC {\\cyrshch }", "\\IeC {\\cyrhrdsn }", "\\IeC {\\cyrery }",
    "\\IeC {\\cyrsftsn }", "\\IeC {\\cyrerev }",
    "\\IeC {\\cyryu }", "\\IeC {\\cyrya }",
};

static const int NUM_RUSSIANS =
    sizeof(TeX_Russians) / sizeof(const char * const);

static int TeX_Russians_Len[NUM_RUSSIANS + 1];

static const char Russians[] = {
    ""
    ""
    ""
    ""
    ""
    ""
    ""
    ""
};

static void printHelp();

int main(int argc, char* argv[]) {
    char line[1024];
    int i;

    if (argc < 3) {
        printHelp();
        return 1;
    }
    FILE* input = fopen(argv[1], "rb");
    if (input == 0) {
        perror("Cannot open input file");
        return 1;
    }
    FILE* output = fopen(argv[2], "wb");
    if (input == 0) {
        perror("Cannot open output file");
        return 1;
    }

    // Initialize TeX_Russians_Len
    for (i = 0; i < NUM_RUSSIANS; ++i) {
        TeX_Russians_Len[i] = strlen(TeX_Russians[i]);
    }

    while (fgets(line, 1024, input) != 0) {
        int len = strlen(line);
        i = 0;
        while (i < len) {
            int j;
            for (j = 0; j < NUM_RUSSIANS; ++j) {
                //... int texLen = strlen(TeX_Russians[j]);
                int texLen = TeX_Russians_Len[j];
                if (
                    len - i >= texLen &&
                    memcmp(
                        line+i, TeX_Russians[j], texLen
                    ) == 0
                ) {
                    // Found TeX Russian letter. Replace it
                    fputc(Russians[j], output);
                    // Skip TeX Russian
                    i += texLen;
                    goto LSkip;
                }
            } // end for

            // A TeX Russian letter was not found.
            // Just write a characted to output
            fputc(line[i], output);
            ++i;

            LSkip: ;
        } // end while (i < len)
    } // end while (fgets...

    fclose(output);
    fclose(input);
    return 0;
}

static void printHelp() {
    printf(
        "Make Russian index from TeX idx-file.\n"
        "Must be performed BEFORE makeindex!\n"
        "The program replaces the Russian letters,\n"
        "produced by TeX in the style \"\\IeC {\\cyra }\",\n"
        "by normal Russian letters in Windows CP-1251\n"
        "encoding. Windows is choosed, because\n"
        "the codes of Russian letters\n"
        "are ordered in Windows encoding.\n\n"
    );
    printf(
        "Usage:\n"
        "    rusind input-file output-file\n"
    );
}
